const URI   = require('url');
const http  = require('http');
const https = require('https');
/**
 * [NPM description]
 * @param {[type]} options [description]
 */
function NPM(options){
  if(!(this instanceof NPM))
    return new NPM(options); 
  return Object.assign(this, options);
}

NPM.prototype.request = (url, params = {}) => {
  const { registry = 'https://registry.npmjs.com' } = this;
  if(url.startsWith('/')) url = registry + url;
  Object.assign(params, URI.parse(url));
  return new Promise((resolve, reject) => {
    const parseBody = (data) => {
      resolve({
        blob: () => data,
        text: () => data + '',
        json: () => JSON.parse(data + '')
      });
    };
    (params.protocol === 'http:' ? http : https)
    .request(params, res => { 
      const buffer = [];
      res
        .on('data', chunk => buffer.push(chunk))
        .on('end', () => parseBody(Buffer.concat(buffer)));
    }).end(params.body);
  });
};

/**
 * [getPackage description]
 * @param  {[type]} name    [description]
 * @param  {[type]} version [description]
 * @return {[type]}         [description]
 */
NPM.prototype.fetch = function(name, version = '') {
  return this
    .request(`/${name}/${version}`)
    .then(res => res.json())
    .then(package => package.name && package);
};

/**
 * [Server description]
 * @type {[type]}
 */
NPM.Proxy = require('./proxy');
NPM.Server = require('./server');
NPM.Storage = require('./storage');
NPM.createServer = function(options){
  return new NPM.Server(options);
};

module.exports = NPM;